﻿
#region Header

/* /////////////////////////////////////////////////////////////////////////////////////////////////
XpVRControllers for Virt-A-Mate by Fabio Den.

Written in Visual Studio
	
Version 1.0 2022-03-09
	Initial release.

License: CC BY-SA
	
///////////////////////////////////////////////////////////////////////////////////////////////// */

using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using UnityEngine.UI;
using UnityEngine.Events;
using UnityEngine.Animations;
using System.Linq;
using MVR.FileManagementSecure;
using SimpleJSON;

#endregion

public class VRControllers : MVRScript
{

	#region Variables

	private JSONStorableBool enabledChk;
	float tolerance = 0.1f;
	float tolerancer = 0.35f;
	private bool changing = false;
	private bool pressing = false;
	
	#endregion

	#region Events

	public override void Init()
	{
		SetupGUI();
	}

	public void OnEnable()
	{

	}

	public void OnDisable()
	{

	}

	public void OnDestroy()
	{
		if (enabledChk.val)
		{
			SuperController.singleton.ToggleDisableAllNavigation();
		}
	}

	#endregion

	#region GUI

	private void SetupGUI()
	{
		enabledChk = GUICreateCheckbox(this, "Enabled", "Enabled", false, false, SyncEnabled);
	}

	public static JSONStorableBool GUICreateCheckbox(MVRScript dll, string name, string label, bool startvalue, bool rightside, JSONStorableBool.SetJSONBoolCallback callback)
	{
		JSONStorableBool gui = new JSONStorableBool(name, startvalue, callback);
		var Toggle = dll.CreateToggle(gui, rightside);
		Toggle.label = label;
		dll.RegisterBool(gui);

		return gui;
	}

	private void SyncEnabled(JSONStorableBool flag)
	{
		enabledChk.val = flag.val;

		pressing = false;
		SuperController.singleton.ToggleDisableAllNavigation();
	}

	#endregion

	#region	Movement

	public void ControllerMove(int dir, float up)
	{
		SuperController sc = SuperController.singleton;
		FreeControllerV3 controller = SuperController.singleton.GetSelectedController();
		if (controller == null) { return; }

		var direction = Vector3.right;

		switch (dir)
		{
			case 1:
				direction = Vector3.right;
				break;
			case 2:
				direction = Vector3.up;
				break;
			case 3:
				direction = Vector3.forward;
				break;
		}

		controller.transform.Translate(direction * Time.deltaTime * up * 0.35f, Camera.main.transform); // Space.Self
	}

	#endregion

	#region	Rotation

	public void ControllerRotate(int dir, float up, int coordinates)
	{
		var rotate = 360 * Time.deltaTime * up * 0.25f;

		SuperController sc = SuperController.singleton;
		FreeControllerV3 target = SuperController.singleton.GetSelectedController();
		if (target == null) { return; }

		switch (coordinates)
		{
			case 1:
				if (dir == 1) { target.transform.RotateAround(target.transform.position, Camera.main.transform.right, -rotate); } // yes
				else if (dir == 2) { target.transform.RotateAround(target.transform.position, Camera.main.transform.up, rotate); } // no
				else if (dir == 3) { target.transform.RotateAround(target.transform.position, Camera.main.transform.forward, rotate); } // roll
				break;
			case 2:
				if (dir == 1) { target.transform.Rotate(rotate, 0, 0, Space.Self); } // yes
				else if (dir == 2) { target.transform.Rotate(0, rotate, 0, Space.Self); }// no
				else if (dir == 3) { target.transform.Rotate(0, 0, rotate, Space.Self); } // roll
				break;
			case 3:
				if (dir == 1) { target.transform.Rotate(rotate, 0, 0, Space.World); } // yes
				else if (dir == 2) { target.transform.Rotate(0, rotate, 0, Space.World); }// no
				else if (dir == 3) { target.transform.Rotate(0, 0, rotate, Space.World); } // roll
				break;
		}
	}

	#endregion

	#region Update

	void FixedUpdate()
	{
		DoStep();
	}

	private void DoStep()
	{
		bool a = SuperController.singleton.GetLeftGrabVal() > tolerance;
		bool b = SuperController.singleton.GetRightGrabVal() > tolerance;

		if (a & b)
		{
			if (changing == false)
			{
				changing = true;
				enabledChk.val = !enabledChk.val;
			}

			return;
		}

		changing = false;

		if (enabledChk.val)
		{
			Vector4 axes = SuperController.singleton.GetFreeNavigateVector(SuperController.singleton.freeMoveAction);
			pressing = a || b;

			if (pressing)
			{
				if (Math.Abs(axes.y) > tolerance) { ControllerMove(3, axes.y); }
				if (Math.Abs(axes.z) > tolerancer)
				{
					ControllerRotate(3, -axes.z, 1);
					return;
				}
			}
			else
			{
				if (Math.Abs(axes.x) > tolerance) { ControllerMove(1, axes.x); }
				if (Math.Abs(axes.y) > tolerance) { ControllerMove(2, axes.y); }
				if (Math.Abs(axes.z) > tolerancer)
				{
					ControllerRotate(2, -axes.z, 1);
					return;
				}
				if (Math.Abs(axes.w) > tolerancer)
				{
					ControllerRotate(1, -axes.w, 1);
					return;
				}
			}
		}
	}

	#endregion

} // End class

